@@ -44,15 +44,6 @@ class InferenceEngine(Module):
         self.quantize_merge_count = 1
         self.quantization_scales = None
 
-        if self.mpu:
-            self.mp_world_size = dist.get_world_size(
-                group=self.mpu.get_model_parallel_group())
-            self.mp_group = self.mpu.get_model_parallel_group()
-        elif self.mp_world_size > 1 and not dist.is_initialized():
-            self._create_model_parallel_group()
-        else:
-            self.module.to(torch.cuda.current_device())
-
         self._check_quantize_setting(quantization_setting)
 
         if self.checkpoint:
@@ -62,6 +53,13 @@ class InferenceEngine(Module):
         if self.dtype:
             self._convert_to_dtype()
 
+        if self.mpu:
+            self.mp_world_size = dist.get_world_size(
+                group=self.mpu.get_model_parallel_group())
+            self.mp_group = self.mpu.get_model_parallel_group()
+        elif self.mp_world_size > 1 and not dist.is_initialized():
+            self._create_model_parallel_group()
+
         # apply injection policy
         if self.injection_dict:
             for client_module, injection_policy in self.injection_dict.items():
@@ -69,6 +67,8 @@ class InferenceEngine(Module):
         elif replace_method == 'auto':
             self._apply_injection_policy()
 
+        self.module.to(torch.cuda.current_device())
+
         if self.mp_world_size > 1:
             self.model_orig_fwd = self.module.forward
             self.module.forward = self.forward
@@ -96,11 +96,6 @@ class InferenceEngine(Module):
         ranks = [i for i in range(self.mp_world_size)]
         self.mp_group = dist.new_group(ranks)
 
-        self.module.to(torch.cuda.current_device())
-        for p in self.module.parameters():
-            if torch.is_tensor(p):
-                dist.broadcast(p, 0)
-
     def _check_quantize_setting(self, quantization_setting):
         self.quatize_bits = 8
         self.mlp_extra_grouping = False
@@ -188,18 +183,9 @@ class InferenceEngine(Module):
         for input in inputs:
             if torch.is_tensor(input):
                 input = input.to(torch.cuda.current_device())
-                if self.mp_world_size > 1:
-                    if not input.is_contiguous():
-                        input = input.contiguous()
-                    dist.broadcast(input, 0)
-
         for k in kwargs:
             if torch.is_tensor(kwargs[k]):
                 kwargs[k] = kwargs[k].to(torch.cuda.current_device())
-                if self.mp_world_size > 1:
-                    if not kwargs[k].is_contiguous():
-                        kwargs[k] = kwargs[k].contiguous()
-                    dist.broadcast(kwargs[k], 0)
 
     def forward(self, *inputs, **kwargs):
         """Execute forward propagation
@@ -213,18 +199,16 @@ class InferenceEngine(Module):
                 for input in inputs:
                     if torch.is_tensor(input):
                         input = input.to(torch.cuda.current_device())
-                        if self.mp_world_size > 1:
-                            if not input.is_contiguous():
-                                input = input.contiguous()
-                            dist.broadcast(input, 0)
-
+                        if not input.is_contiguous():
+                            input = input.contiguous()
                 for k in kwargs:
                     if torch.is_tensor(kwargs[k]):
                         kwargs[k] = kwargs[k].to(torch.cuda.current_device())
-                        if self.mp_world_size > 1:
-                            if not kwargs[k].is_contiguous():
-                                kwargs[k] = kwargs[k].contiguous()
-                            dist.broadcast(kwargs[k], 0)
+                        if not kwargs[k].is_contiguous():
+                            kwargs[k] = kwargs[k].contiguous()
+                        dist.broadcast(kwargs[k], 0)
 
-            return self.model_orig_fwd(*inputs, **kwargs)
-        return self.module(*inputs, **kwargs)
+            outputs = self.model_orig_fwd(*inputs, **kwargs)
+        else:
+            outputs = self.module(*inputs, **kwargs)
+        return outputs
