@@ -17,6 +17,7 @@ from torch import Tensor
 from torch.nn.parameter import Parameter
 from torch.nn import init
 from torch.nn.modules.module import Module
+from deepspeed.runtime.utils import noop_decorator
 
 tensor_map = {}
 
@@ -26,10 +27,19 @@ def print_rank_0(message, debug=False, force=False):
         print(message)
 
 
+try:
+    autocast_custom_fwd = torch.cuda.amp.custom_fwd
+    autocast_custom_bwd = torch.cuda.amp.custom_bwd
+except (ImportError, AttributeError) as exp:
+    autocast_custom_fwd = noop_decorator
+    autocast_custom_bwd = noop_decorator
+
+
 class LinearFunctionForZeroStage3(torch.autograd.Function):
 
     # Note that both forward and backward are @staticmethods
     @staticmethod
+    @autocast_custom_fwd
     # bias is an optional argument
     def forward(ctx, input, weight, bias=None):
         #print("In ZeRO Linear Function")
@@ -56,6 +66,7 @@ class LinearFunctionForZeroStage3(torch.autograd.Function):
 
     # This function has only a single output, so it gets only one gradient
     @staticmethod
+    @autocast_custom_bwd
     def backward(ctx, grad_output):
         # This is a pattern that is very convenient - at the top of backward
         # unpack saved_tensors and initialize all gradients w.r.t. inputs to
